`resetall

module Gowin_AHB_Arbiter_template (
  LOCKUP,
  HALTED,
  JTAG_3,   //trstn
  JTAG_4,   //NC
  JTAG_5,   //tdi
  JTAG_6,   //NC
  JTAG_7,   //tms   |       swdio
  JTAG_8,   //extFlash_sspi2_sio3/holdn
  JTAG_9,   //tck   |       swclk
  JTAG_10,  //extFlash_spi2_mosi/sio1
  JTAG_11,  //NC    |       
  JTAG_12,  //extFlash_spi2_sclk
  JTAG_13,  //tdo   |       swo
  JTAG_14,  //extFlash_spi2_miso/sio0
  JTAG_15,  //srstn
  JTAG_16,  //extFlash_spi2_cs
  JTAG_17,  //NC
  JTAG_18,  //extFlash_sspi2_sio2/WPn
  UART0RXD,
  UART0TXD,
  FLASH_SPI_HOLDN,
  FLASH_SPI_CSN,
  FLASH_SPI_MISO,
  FLASH_SPI_MOSI,
  FLASH_SPI_WPN,
  FLASH_SPI_CLK,
  HCLK,		//System Clock
  hwRstn	//System Reset
);

//JTAG
input JTAG_3;    //trstn
output JTAG_4;   //NC
input JTAG_5;    //tdi
output JTAG_6;   //NC
inout JTAG_7 /* synthesis syn_tristate = 1 */ ;
output JTAG_8;   //extFlash_sspi2_sio3/holdn
input JTAG_9;    //tck
output JTAG_10;  //extFlash_spi2_mosi/sio1
output JTAG_11;  //NC RTCK
output JTAG_12;  //extFlash_spi2_sclk
output JTAG_13;  //tdo
output JTAG_14;  //extFlash_spi2_miso/sio0
output JTAG_15;  //srstn
output JTAG_16;  //extFlash_spi2_cs
output JTAG_17;  //NC DBGQ
output JTAG_18;

input HCLK;          // System clock
input hwRstn;        // System reset

//UART0
input UART0RXD ;
output UART0TXD ;

output LOCKUP;        // Core is in lockup state
output HALTED;        // Core is in Halt debug state

//SPI-Flash
inout FLASH_SPI_HOLDN /* synthesis syn_tristate = 1 */ ;
inout FLASH_SPI_CSN /* synthesis syn_tristate = 1 */ ;
inout FLASH_SPI_MISO /* synthesis syn_tristate = 1 */ ;
inout FLASH_SPI_MOSI /* synthesis syn_tristate = 1 */ ;
inout FLASH_SPI_WPN /* synthesis syn_tristate = 1 */ ;
inout FLASH_SPI_CLK /* synthesis syn_tristate = 1 */ ;

//AHB Bus IF
wire [31:0] AHB2HRDATA;      // Data from slave to master
wire        AHB2HREADYOUT;   // Slave ready signal
wire [1:0]  AHB2HRESP;       // Slave response signal  
wire [1:0]  AHB2HTRANS;      // Transfer type
wire [2:0]  AHB2HBURST;      // Burst type
wire [3:0]  AHB2HPROT;       // Transfer protection bits
wire [2:0]  AHB2HSIZE;       // Transfer size
wire        AHB2HWRITE;      // Transfer direction
wire        AHB2HMASTLOCK;   // Transfer is a locked transfer
wire        AHB2HREADYMUX;   // Ready mux signal
wire [3:0]  AH2BHMASTER;     // Transfer is a locked transfer
wire [31:0] AHB2HADDR;       // Transfer address
wire [31:0] AHB2HWDATA;      // Data from master to slave
wire        AHB2HSEL;        // Select
wire        AHB2HCLK;        // Bus clock signal
wire        AHB2HRESET;      // Bus reset signal

wire [31:0] SHRDATAM0 ;
wire        SHREADYOUTM0 ;
wire [1:0]  SHRESPM0 ;
wire        SHSELM0 ;
wire [31:0] SHADDRM0 ;
wire [1:0]  SHTRANSM0 ;
wire        SHWRITEM0 ;
wire [2:0]  SHSIZEM0 ;
wire [2:0]  SHBURSTM0 ;
wire [3:0]  SHPROTM0 ;
wire [3:0]  SHMASTERM0 ;
wire [31:0] SHWDATAM0 ;
wire        SHMASTLOCKM0 ;
wire        SHREADYMUXM0 ;

//---------------Gowin_EMPU_M1 connects with AHB Bus Arbiter---------------------//
//Gowin_EMPU_M1
Gowin_EMPU_M1_Top M1_template (
  .LOCKUP(LOCKUP),
  .HALTED(HALTED),
  .JTAG_3(JTAG_3),   //trstn
  .JTAG_4(JTAG_4),   //NC
  .JTAG_5(JTAG_5),   //tdi
  .JTAG_6(JTAG_6),   //NC
  .JTAG_7(JTAG_7),   //tms   |       swdio
  .JTAG_8(JTAG_8),   //extFlash_sspi2_sio3/holdn
  .JTAG_9(JTAG_9),   //tck   |       swclk
  .JTAG_10(JTAG_10),  //extFlash_spi2_mosi/sio1
  .JTAG_11(JTAG_11),  //NC    |       
  .JTAG_12(JTAG_12),  //extFlash_spi2_sclk
  .JTAG_13(JTAG_13),  //tdo   |       swo
  .JTAG_14(JTAG_14),  //extFlash_spi2_miso/sio0
  .JTAG_15(JTAG_15),  //srstn
  .JTAG_16(JTAG_16),  //extFlash_spi2_cs
  .JTAG_17(JTAG_17),  //NC
  .JTAG_18(JTAG_18),  //extFlash_sspi2_sio2/WPn
  .UART0RXD(UART0RXD),
  .UART0TXD(UART0TXD),
  .AHB2HRDATA(AHB2HRDATA),       // Data from slave to master
  .AHB2HREADYOUT(AHB2HREADYOUT), // Slave ready signal, from slave, 1'b1
  .AHB2HRESP(AHB2HRESP),         // Slave response signal  
  .AHB2HTRANS(AHB2HTRANS),       // Transfer type
  .AHB2HBURST(AHB2HBURST),       // Burst type
  .AHB2HPROT(AHB2HPROT),         // Transfer protection bits
  .AHB2HSIZE(AHB2HSIZE),         // Transfer size
  .AHB2HWRITE(AHB2HWRITE),       // Transfer direction
  .AHB2HREADYMUX(AHB2HREADYMUX), // Ready mux
  .AHB2HMASTLOCK(AHB2HMASTLOCK), // Transfer is a locked transfer
  .AHB2HMASTER(AH2BHMASTER),     // Transfer is a locked transfer
  .AHB2HADDR(AHB2HADDR),         // Transfer address
  .AHB2HWDATA(AHB2HWDATA),       // Data from master to slave   AHB2HSEL,
  .AHB2HSEL(AHB2HSEL),           // Select
  .AHB2HCLK(AHB2HCLK),           // Clock
  .AHB2HRESET(AHB2HRESET),       // Reset
  .FLASH_SPI_HOLDN(FLASH_SPI_HOLDN),
  .FLASH_SPI_CSN(FLASH_SPI_CSN),
  .FLASH_SPI_MISO(FLASH_SPI_MISO),
  .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
  .FLASH_SPI_WPN(FLASH_SPI_WPN),
  .FLASH_SPI_CLK(FLASH_SPI_CLK),
  .HCLK(HCLK),		//System Clock
  .hwRstn(hwRstn)	//System Reset
);

//---------------AHB Bus Arbiter connects with Gowin_EMPU_M1 and Multiple---------------------//
//Gowin_AHB_Arbiter
Gowin_AHB_Arbiter_Top Arbiter_template 
(
  .HCLK(AHB2HCLK),
  .HRESETn(AHB2HRESET),
  .MHSELS0(AHB2HSEL),
  .MHADDRS0(AHB2HADDR),
  .MHTRANSS0(AHB2HTRANS),
  .MHWRITES0(AHB2HWRITE),
  .MHSIZES0(AHB2HSIZE),
  .MHBURSTS0(AHB2HBURST),
  .MHPROTS0(AHB2HPROT),
  .MHMASTERS0(AH2BHMASTER),
  .MHWDATAS0(AHB2HWDATA),
  .MHMASTLOCKS0(AHB2HMASTLOCK),
  .MHREADYS0(AHB2HREADYMUX),
  .MHRDATAS0(AHB2HRDATA),
  .MHREADYOUTS0(AHB2HREADYOUT),
  .MHRESPS0(AHB2HRESP),
  .SHRDATAM0(SHRDATAM0),
  .SHREADYOUTM0(SHREADYOUTM0),
  .SHRESPM0(SHRESPM0),
  .SHSELM0(SHSELM0),
  .SHADDRM0(SHADDRM0),
  .SHTRANSM0(SHTRANSM0),
  .SHWRITEM0(SHWRITEM0),
  .SHSIZEM0(SHSIZEM0),
  .SHBURSTM0(SHBURSTM0),
  .SHPROTM0(SHPROTM0),
  .SHMASTERM0(SHMASTERM0),
  .SHWDATAM0(SHWDATAM0),
  .SHMASTLOCKM0(SHMASTLOCKM0),
  .SHREADYMUXM0(SHREADYMUXM0)
);

//Gowin_AHB_Multiple
Gowin_AHB_Multiple u_ahb_multiple
(
	.AHB_HRDATA(SHRDATAM0),
	.AHB_HREADY(SHREADYOUTM0),//ready signal, slave to MCU master, 1'b1
	.AHB_HRESP(SHRESPM0),//respone signal, slave to MCU master
	.AHB_HTRANS(SHTRANSM0),
	.AHB_HBURST(SHBURSTM0),
	.AHB_HPROT(SHPROTM0),
	.AHB_HSIZE(SHSIZEM0),
	.AHB_HWRITE(SHWRITEM0),
	.AHB_HMASTLOCK(SHMASTLOCKM0),
	.AHB_HMASTER(SHMASTERM0),
	.AHB_HADDR(SHADDRM0[11:0]),
	.AHB_HWDATA(SHWDATAM0),
	.AHB_HSEL(SHSELM0),
	.AHB_HCLK(AHB2HCLK),
	.AHB_HRESETn(AHB2HRESET)
);

endmodule